package com.wrj.driver.server.controller;

import com.wrj.driver.server.config.WechatPayConfig;
import com.wrj.driver.server.constant.Constants;
import com.wrj.driver.server.entity.OrderPayInfo;
import com.wrj.driver.server.entity.TdTenantApplet;
import com.wrj.driver.server.exception.BusinessException;
import com.wrj.driver.server.interceptor.TenantContextHolder;
import com.wrj.driver.server.response.BaseResponse;
import com.wrj.driver.server.service.IOrderPayInfoService;
import com.wrj.driver.server.service.ITdTenantAppletService;
import com.wrj.driver.server.util.InputStreamUtil;
import com.wrj.driver.server.vo.H5PayVo;
import com.wechat.pay.java.core.Config;
import com.wechat.pay.java.core.RSAAutoCertificateConfig;
import com.wechat.pay.java.core.exception.ServiceException;
import com.wechat.pay.java.core.util.PemUtil;
import com.wechat.pay.java.service.payments.h5.H5Service;
import com.wechat.pay.java.service.payments.h5.model.*;
import com.wechat.pay.java.service.payments.model.Transaction;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.math.BigDecimal;

/**
 * 微信支付小程序 接口
 */
@Api(tags = "H5支付")
@RestController
@RequestMapping("/H5/pay")
@Slf4j
public class H5PayController {

    @Resource
    private WechatPayConfig tdTenantApplet;

    @Autowired
    private IOrderPayInfoService orderPayInfoService;

    @Autowired
    private ITdTenantAppletService tdTenantAppletService;

    //生成预支付订单
    @ApiOperation("生成预支付订单")
    @PostMapping("/prepay")
    public BaseResponse createPrepay(@RequestBody H5PayVo payVo){
        // 构建service
        payVo.setTenantId(TenantContextHolder.getTenantId() == null ? Constants.DEFAULT_TENANT_ID : TenantContextHolder.getTenantId());
        //查询租户商户信息
        TdTenantApplet tdTenantApplet = tdTenantAppletService.getById(payVo.getTenantId());
        if (tdTenantApplet == null){
            throw new BusinessException( "商户信息不存在，请联系服务人员");
        }

        H5Service service = createService(tdTenantApplet);
        //在order_pay_info里新增一条数据
        OrderPayInfo orderPayInfo = orderPayInfoService.createOrderPayInfo(payVo.getMoney(),payVo.getUserId(),payVo.getDescription(), payVo.getTradeType(), payVo.getOutTradeNo(), "h5", payVo.getDetailId(), payVo.getTenantId());

        // 请求下单参数
        PrepayRequest request = new PrepayRequest();
        Amount amount = new Amount();
        amount.setTotal(Integer.valueOf((payVo.getMoney().multiply(new BigDecimal("100")).setScale(0).toString())+""));
        request.setAmount(amount);
        request.setAppid(tdTenantApplet.getAppId());
        request.setMchid(tdTenantApplet.getMchId());
        request.setDescription(payVo.getDescription());
        request.setNotifyUrl(tdTenantApplet.getPayNoticeUrl());
        request.setOutTradeNo(orderPayInfo.getPayId().toString());
        //场景参数
        SceneInfo sceneInfo = new SceneInfo();
        sceneInfo.setPayerClientIp(payVo.getClientIp());
        request.setSceneInfo(sceneInfo);

        // 调用下单方法，得到应答
        PrepayResponse response = service.prepay(request);
        // 使用微信扫描 code_url 对应的二维码，即可体验Native支付
//        log.info(response);
        return BaseResponse.success(response);
    }

    private H5Service createService(TdTenantApplet tdTenantApplet) {
        Config config =
                new RSAAutoCertificateConfig.Builder()
                        .merchantId(tdTenantApplet.getMchId())
                        .privateKey(PemUtil.loadPrivateKeyFromString(InputStreamUtil.getContentStr(tdTenantApplet.getPrivateKeyPath())))
//                        .privateKeyFromPath(tdTenantApplet.getPrivateKeyPath())
                        .merchantSerialNumber(tdTenantApplet.getMchSerialNo())
                        .apiV3Key(tdTenantApplet.getApiV3Key())
                        .build();
        H5Service service = new H5Service.Builder().config(config).build();

        return service;
    }


    //查询订单接口
    @ApiOperation("查询支付订单")
    @PostMapping("/queryOrder")
    public BaseResponse queryOrder(@RequestBody H5PayVo payVo){
        payVo.setTenantId(TenantContextHolder.getTenantId() == null ? Constants.DEFAULT_TENANT_ID : TenantContextHolder.getTenantId());
        //查询租户商户信息
        TdTenantApplet tdTenantApplet = tdTenantAppletService.getById(payVo.getTenantId());
        if (tdTenantApplet == null){
            throw new BusinessException( "商户信息不存在，请联系服务人员");
        }
        //获取openId
        QueryOrderByOutTradeNoRequest queryRequest = new QueryOrderByOutTradeNoRequest();
        queryRequest.setMchid(tdTenantApplet.getMchId());
        //支付订单id
        queryRequest.setOutTradeNo(payVo.getOutTradeNo());
        H5Service service = createService(tdTenantApplet);

        try {
            Transaction result = service.queryOrderByOutTradeNo(queryRequest);
            System.out.println(result.getTradeState());
            return BaseResponse.success(result);
        } catch (ServiceException e) {
            // API返回失败, 例如ORDER_NOT_EXISTS
            System.out.printf("code=[%s], message=[%s]\n", e.getErrorCode(), e.getErrorMessage());
            System.out.printf("reponse body=[%s]\n", e.getResponseBody());
            return  BaseResponse.fail(e.getErrorMessage());
        }

    }

    //关闭订单接口
    @ApiOperation("关闭支付订单")
    @PostMapping("/closeOrder")
    public BaseResponse closeOrder(@RequestBody H5PayVo payVo){
        payVo.setTenantId(TenantContextHolder.getTenantId() == null ? Constants.DEFAULT_TENANT_ID : TenantContextHolder.getTenantId());
        //查询租户商户信息
        TdTenantApplet tdTenantApplet = tdTenantAppletService.getById(payVo.getTenantId());
        if (tdTenantApplet == null){
            throw new BusinessException( "商户信息不存在，请联系服务人员");
        }
        H5Service service = createService(tdTenantApplet);
        CloseOrderRequest closeRequest = new CloseOrderRequest();
        closeRequest.setMchid(tdTenantApplet.getMchId());
        closeRequest.setOutTradeNo(payVo.getOutTradeNo());
        // 方法没有返回值，意味着成功时API返回204 No Content
        service.closeOrder(closeRequest);
        return BaseResponse.success();
    }
}
