package com.wrj.driver.server.controller;


import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONObject;
import com.wrj.driver.server.entity.OrderPayInfo;
import com.wrj.driver.server.entity.TdTenantApplet;
import com.wrj.driver.server.enums.PurchaseProductTypeEnum;
import com.wrj.driver.server.exception.BusinessException;
import com.wrj.driver.server.service.IOrderPayInfoService;
import com.wrj.driver.server.service.ITdSysUserMemberService;
import com.wrj.driver.server.service.ITdTenantAppletService;
import com.wrj.driver.server.util.InputStreamUtil;
import com.wechat.pay.java.core.RSAAutoCertificateConfig;
import com.wechat.pay.java.core.RSAPublicKeyConfig;
import com.wechat.pay.java.core.notification.NotificationConfig;
import com.wechat.pay.java.core.notification.NotificationParser;
import com.wechat.pay.java.core.notification.RequestParam;
import com.wechat.pay.java.core.util.PemUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * <p>
 * 支付回调日志表 前端控制器
 * </p>
 *
 * @author Automated procedures
 * @since 2023-08-10
 */
@Api(tags = "支付回调")
@RestController
@RequestMapping("/payNoticeLog")
@Slf4j
public class PayNoticeLogController {
//    @Resource
//    private WechatPayConfig tdTenantApplet;

    @Autowired
    private ITdSysUserMemberService userMemberService;

    @Autowired
    private IOrderPayInfoService payInfoService;

    @Autowired
    private ITdTenantAppletService tdTenantAppletService;

    /**
     * 寻驾小程序 支付回调接口 本接口不能做验证
     * @param wechatPayCertificateSerialNumber
     * @param signature
     * @param timstamp
     * @param nonce
     * @param requestBody
     * @return
     */
    @ApiOperation("支付回调接口")
    @PostMapping("/{tenantId}")
    public JSONObject payNotice(@RequestHeader("Wechatpay-Serial") String wechatPayCertificateSerialNumber,
                                  @RequestHeader("Wechatpay-Signature") String signature,
                                  @RequestHeader("Wechatpay-Timestamp") String timstamp,
                                  @RequestHeader("Wechatpay-Nonce") String nonce,
                                  @RequestBody String requestBody,
                                  @PathVariable("tenantId") Long tenantId){
        log.info("支付回调信息==============》{}",requestBody);
        log.info("支付回调信息==============》tenantId{}",tenantId);

        //查询租户id
        TdTenantApplet tdTenantApplet = null;
        if (tenantId != null){
            //查询租户商户信息
            tdTenantApplet = tdTenantAppletService.getById(tenantId);
        }

        if (tdTenantApplet == null){
            log.info("支付回调==============》商户信息不存在");
            throw new BusinessException( "商户信息不存在，请联系服务人员");
        }

        NotificationConfig config = null;
        if (tdTenantApplet.getIsNew()){
            //新支付
            config = new RSAPublicKeyConfig.Builder()
                    .merchantId(tdTenantApplet.getMchId())
                    .privateKeyFromPath(tdTenantApplet.getPrivateKeyPath())
                    .publicKeyFromPath(tdTenantApplet.getPubKeyPath())
                    .publicKeyId(tdTenantApplet.getPubKeyId())
                    .merchantSerialNumber(tdTenantApplet.getMchSerialNo())
                    .apiV3Key(tdTenantApplet.getApiV3Key())
                    .build();
        } else {
            //老支付
            config = new RSAAutoCertificateConfig.Builder()
                .merchantId(tdTenantApplet.getMchId())
//                .privateKey(PemUtil.loadPrivateKeyFromString(InputStreamUtil.getContentStr(tdTenantApplet.getPrivateKeyPath())))
                .privateKeyFromPath(tdTenantApplet.getPrivateKeyPath())
                .merchantSerialNumber(tdTenantApplet.getMchSerialNo())
                .apiV3Key(tdTenantApplet.getApiV3Key())
                    .build();

        }

        RequestParam requestParam = new RequestParam.Builder()
                .serialNumber(wechatPayCertificateSerialNumber)
                .nonce(nonce)
                .signature(signature)
                .timestamp(timstamp)
                // 若未设置signType，默认值为 WECHATPAY2-SHA256-RSA2048
                .body(requestBody)
                .build();

        // 初始化 NotificationParser
        NotificationParser parser = new NotificationParser(config);


        // 验签并解密报文
        JSONObject decryptObject = parser.parse(requestParam,JSONObject.class);
        System.out.println("decryptObject="+decryptObject.toJSONString());

        String trade_state=decryptObject.getString("trade_state");
        //取业务id
        String outTradeNo = decryptObject.getString("out_trade_no");
        JSONObject jsonResponse = new JSONObject();
        OrderPayInfo payInfo = payInfoService.getById(Long.parseLong(outTradeNo));
        if(trade_state.equals("SUCCESS")) {
            //各种业务逻辑
            //1.订单详情表里修改数据
            payInfoService.payNotice(payInfo,Boolean.TRUE);
            //2.如果购买商品是会员则冲会员，其余的则不做操作
            if (StrUtil.equals(payInfo.getPurchaseProductType(), PurchaseProductTypeEnum.MEMBER.getValue())){
                userMemberService.purchaseMember(payInfo);
            }
        }else{
            //还是各种业务逻辑
            //1.订单详情表里修改数据
            payInfoService.payNotice(payInfo,Boolean.FALSE);
        }
        jsonResponse.put("code", "SUCCESS");
        jsonResponse.put("message", "成功");
        return jsonResponse;
    }


    @ApiOperation("通关考支付回调接口")
    @PostMapping("/tgk")
    public JSONObject tGKPayNotice(@RequestHeader("Wechatpay-Serial") String wechatPayCertificateSerialNumber,
                                @RequestHeader("Wechatpay-Signature") String signature,
                                @RequestHeader("Wechatpay-Timestamp") String timstamp,
                                @RequestHeader("Wechatpay-Nonce") String nonce,
                                @RequestBody String requestBody){
        log.info("支付回调信息==============》{}",requestBody);
        //查询租户id
        Long tenantId = 51710003l;
        TdTenantApplet tdTenantApplet = tdTenantAppletService.getById(tenantId);

        if (tdTenantApplet == null){
            throw new BusinessException( "商户信息不存在，请联系服务人员");
        }

        NotificationConfig config = new RSAAutoCertificateConfig.Builder()
                .merchantId(tdTenantApplet.getMchId())
                .privateKey(PemUtil.loadPrivateKeyFromString(InputStreamUtil.getContentStr(tdTenantApplet.getPrivateKeyPath())))
//                .privateKeyFromPath(tdTenantApplet.getPrivateKeyPath())
                .merchantSerialNumber(tdTenantApplet.getMchSerialNo())
                .apiV3Key(tdTenantApplet.getApiV3Key())
                .build();
        RequestParam requestParam = new RequestParam.Builder()
                .serialNumber(wechatPayCertificateSerialNumber)
                .nonce(nonce)
                .signature(signature)
                .timestamp(timstamp)
                // 若未设置signType，默认值为 WECHATPAY2-SHA256-RSA2048

                .body(requestBody)
                .build();

        // 初始化 NotificationParser
        NotificationParser parser = new NotificationParser(config);

        // 验签并解密报文
        JSONObject decryptObject = parser.parse(requestParam,JSONObject.class);
        System.out.println("decryptObject="+decryptObject.toJSONString());

        String trade_state=decryptObject.getString("trade_state");
        //取业务id
        String outTradeNo = decryptObject.getString("out_trade_no");
        JSONObject jsonResponse = new JSONObject();
        OrderPayInfo payInfo = payInfoService.getById(Long.parseLong(outTradeNo));
        if(trade_state.equals("SUCCESS")) {
            //各种业务逻辑
            //1.订单详情表里修改数据
            payInfoService.payNotice(payInfo,Boolean.TRUE);
            //2.如果购买商品是会员则冲会员，其余的则不做操作
            if (StrUtil.equals(payInfo.getPurchaseProductType(), PurchaseProductTypeEnum.MEMBER.getValue())){
                userMemberService.purchaseMember(payInfo);
            }
        }else{
            //还是各种业务逻辑
            //1.订单详情表里修改数据
            payInfoService.payNotice(payInfo,Boolean.FALSE);
        }
        jsonResponse.put("code", "SUCCESS");
        jsonResponse.put("message", "成功");
        return jsonResponse;
    }
}
