package com.wrj.driver.server.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import com.wrj.driver.server.constant.Constants;
import com.wrj.driver.server.dto.MemberQueryDto;
import com.wrj.driver.server.entity.TdMember;
import com.wrj.driver.server.mapper.SysDistributionDetailMapper;
import com.wrj.driver.server.mapper.TdMemberMapper;
import com.wrj.driver.server.mapper.TdSysUserDiscountMapper;
import com.wrj.driver.server.service.ITdMemberService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.wrj.driver.server.util.SecurityUtil;
import com.wrj.driver.server.util.TokenThreadUtil;
import com.wrj.driver.server.vo.MemberVo;
import com.wrj.driver.server.vo.UserDiscountVO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 会员类型表; 服务实现类
 * </p>
 *
 * @author Automated procedures
 * @since 2023-08-10
 */
@Service
@Slf4j
public class TdMemberServiceImpl extends ServiceImpl<TdMemberMapper, TdMember> implements ITdMemberService {

    @Autowired
    private SysDistributionDetailMapper sysDistributionDetailMapper;

    @Autowired
    private TdSysUserDiscountMapper userDiscountMapper;

    @Override
    public List<MemberVo> queryMember(MemberQueryDto queryDto) {

        List<MemberVo> resultList = new ArrayList<>();

        //获取用户已开通且未到期的会员（即使该会员已不售卖了）
        List<MemberVo> userMemberList = new ArrayList<>();
        List<UserDiscountVO> userDiscounts = new ArrayList<>();
        try {
            if (StrUtil.isNotBlank(TokenThreadUtil.getToken())){
                userMemberList = queryUserMember(queryDto, SecurityUtil.getUserId());
                //查询用户折扣
                userDiscounts = userDiscountMapper.selectUserDiscountByUserId(SecurityUtil.getUserId(), queryDto.getTenantId());

//                distributionId = SecurityUtil.getDistributionId();
            }
        }catch (Exception e){
            log.info("===================>用户没有登录");
        }

        Map<Integer, MemberVo> userMemberMap = userMemberList.stream().collect(Collectors.toMap(MemberVo::getMemberId, v -> v, (v1, v2) -> v1));

        Map<String, List<UserDiscountVO>> userDiscountMap = userDiscounts.stream().collect(Collectors.groupingBy(UserDiscountVO::getParam2));

        //将用户购买的已停售但未过期的会员放在最前面
        List<MemberVo> expireMemberlist = userMemberList.stream().filter(s -> StrUtil.equals(s.getIsActive(), Constants.IS_ACTIVE_FALSE)).collect(Collectors.toList());
        if (CollectionUtil.isNotEmpty(expireMemberlist)){
            resultList.addAll(expireMemberlist);
        }

        //获取现在可开通的会员
        List<MemberVo> memberList = this.getBaseMapper().queryMember(queryDto);
//        //根据分销id查询分销信息
//        List<SysDistributionDetail> detailList = null;
//        Map<Long, SysDistributionDetail> distributionDetailMap = null;
//        if(!StringUtil.isNullOrEmpty(distributionId)){
//            detailList = sysDistributionDetailMapper.selectCurrentTimeList(distributionId, DateTimeUtil.getCurrentStr(DateTimeFormatter.ofPattern("yyyy-MM-dd")));
//        }
//
//        if (null != detailList){
//            detailList = detailList.stream().distinct().collect(Collectors.toList());
//            distributionDetailMap = detailList.stream().collect(Collectors.toMap(SysDistributionDetail::getMemberId, v -> v, (v1, v2) -> v1));
//        }



        if (CollectionUtil.isNotEmpty(memberList)){
            for (MemberVo memberVo : memberList) {
                if(userMemberMap.containsKey(memberVo.getMemberId())){
                    resultList.add(userMemberMap.get(memberVo.getMemberId()));
                }
//                else if(null != distributionDetailMap && distributionDetailMap.containsKey(Long.valueOf(memberVo.getMemberId())) ){
//                    SysDistributionDetail detail = distributionDetailMap.get(Long.valueOf(memberVo.getMemberId()));
//                    memberVo.setDiscount(detail.getDiscountedPrice());
//                    memberVo.setDetailId(detail.getDetailId());
//                    resultList.add(memberVo);
//                }
                else {

                    //查看是否有其他的折扣价
                    if (userDiscountMap != null && userDiscountMap.get(memberVo.getMemberId()+"") != null){
                        List<UserDiscountVO> userDiscountVOS = userDiscountMap.get(memberVo.getMemberId()+"");
                        userDiscountVOS = userDiscountVOS.stream().sorted(Comparator.comparing(UserDiscountVO::getDiscount)).collect(Collectors.toList());
                        UserDiscountVO min = userDiscountVOS.get(0);
                        if (min.getDiscount() != null && min.getDiscount().compareTo(memberVo.getDiscount()) <= 0){
                            memberVo.setActualPrice(min.getDiscount());
                            memberVo.setActualEndTime(min.getEndTime() == null ? null : min.getEndTime().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm")));
                        }
                    }

                    resultList.add(memberVo);

                }
            }
        }

        return resultList;
    }

    @Override
    public List<MemberVo> queryUserMember(MemberQueryDto queryDto,Long userId) {
        return this.getBaseMapper().queryUserMember(queryDto,userId);
    }

}
