package com.wrj.driver.server.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.wrj.driver.server.dto.SecurityUser;
import com.wrj.driver.server.dto.TestQueryDto;
import com.wrj.driver.server.entity.*;
import com.wrj.driver.server.service.*;
import com.wrj.driver.server.vo.ExamParam;
import com.wrj.driver.server.vo.TestScoreVO;
import com.wrj.driver.server.dto.TestSubmitDto;
import com.wrj.driver.server.entity.*;
import com.wrj.driver.server.enums.StatusEnum;
import com.wrj.driver.server.exception.BusinessException;
import com.wrj.driver.server.mapper.TdQuestionTestMapper;
import com.wrj.driver.server.response.BaseResponse;
import com.wrj.driver.server.service.*;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.wrj.driver.server.util.SecurityUtil;
import com.wrj.driver.server.vo.MemberVo;
import com.wrj.driver.server.vo.TestTotalVo;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.OptionalDouble;
import java.util.stream.Collectors;

/**
 * <p>
 * 模拟考试成绩表; 服务实现类
 * </p>
 *
 * @author Automated procedures
 * @since 2023-08-10
 */
@Service
public class TdQuestionTestServiceImpl extends ServiceImpl<TdQuestionTestMapper, TdQuestionTest> implements ITdQuestionTestService {

    @Autowired
    private ITdMemberDiscountService memberDiscountService;

    @Autowired
    private ITdMemberService memberService;

    @Autowired
    private ITdSysUserMemberService userMemberService;

    @Autowired
    private ITdSysUserDiscountService userDiscountService;

    @Autowired
    private ITdSysConfigService configService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public BaseResponse testSubmit(TestSubmitDto submitDto) {

        TdQuestionTest questionTest = new TdQuestionTest();
        BeanUtils.copyProperties(submitDto, questionTest);
        questionTest.setUserId(SecurityUtil.getUserId())
                .setCreateTime(LocalDateTime.now());
        boolean save = this.save(questionTest);
        if (!save){
            throw new BusinessException("考试成绩提交失败");
        }
        List<MemberVo> tip = new ArrayList<>();
        //如果考试合格, 赠送会员
        if (submitDto.getScore() >= 90){
            //判断是否首次上传


            List<TdMemberDiscount> memberDiscounts = memberDiscountService.selectList("1", questionTest.getSubject());
            if (memberDiscounts != null){
                for (TdMemberDiscount memberDiscount : memberDiscounts){
                    boolean flag = false;

                    //查询是否已赠送过
                    LambdaQueryWrapper queryWrapper = new LambdaQueryWrapper<TdSysUserDiscount>()
                            .eq(TdSysUserDiscount::getDiscountId, memberDiscount.getId())
                            .eq(TdSysUserDiscount::getUserId, questionTest.getUserId());
                    List<TdSysUserDiscount> userDiscounts = userDiscountService.list(queryWrapper);
                    if (userDiscounts == null || userDiscounts.size() == 0){
                        if (StringUtils.isEmpty(memberDiscount.getParam1())){
                            flag = true;
                        } else {
                            //查询当前用户是由有参数1的会员，如果有则下发折扣
                            List<TdSysUserMember> userMembers = userMemberService.list(new LambdaQueryWrapper<TdSysUserMember>()
                                    .eq(TdSysUserMember::getMemberId, memberDiscount.getParam1())
                                    .eq(TdSysUserMember::getUserId, questionTest.getUserId()));
                            if(userMembers != null && userMembers.size() > 0){
                                flag = true;
                            }
                        }

                        if (flag){
                            //目标折扣
//                    TdMemberDiscount targetDisCOunt = memberDiscountService.selectOne("2", questionTest.getSubject());
                            TdMember member = memberService.getById(Integer.valueOf(memberDiscount.getParam2()));
                            MemberVo model = new MemberVo();
                            BeanUtils.copyProperties(member, model);
                            tip.add(model);
                            TdSysUserDiscount userDiscount = new TdSysUserDiscount()
                                    .setDiscountId(memberDiscount.getId())
                                    .setUserId(questionTest.getUserId())
                                    .setDiscount(memberDiscount.getDiscount())
                                    .setDuration(memberDiscount.getDuration())
                                    .setUnit(memberDiscount.getUnit())
                                    .setOperUser("系统")
                                    .setOperTime(LocalDateTime.now())
                                    .setIsInvalid(0);
                            if ("1".equals(memberDiscount.getUnit())){
                                userDiscount.setEndTime(LocalDateTime.now().plusDays(memberDiscount.getDuration()));
                            } else if ("2".equals(memberDiscount.getUnit())){
                                userDiscount.setEndTime(LocalDateTime.now().plusMonths(memberDiscount.getDuration()));
                            } else if ("3".equals(memberDiscount.getUnit())){
                                userDiscount.setEndTime(LocalDateTime.now().plusYears(memberDiscount.getDuration()));
                            }
                            userDiscountService.save(userDiscount);
                        }
                    }

                }
            }
        }

        return new BaseResponse(StatusEnum.SUCCESS.getCode(), "", questionTest, tip);
    }

    @Override
    public TestTotalVo testTotal(TestQueryDto queryDto) {
        TestTotalVo testTotalVo = new TestTotalVo();
        LambdaQueryWrapper<TdQuestionTest> queryWrapper = new LambdaQueryWrapper<TdQuestionTest>()
                .eq(TdQuestionTest::getCarTypeId, queryDto.getCarTypeId())
                .eq(TdQuestionTest::getSubject, queryDto.getSubject())
                .eq(TdQuestionTest::getUserId, SecurityUtil.getUserId())
                .orderByDesc(TdQuestionTest::getCreateTime);

        List<TdQuestionTest> list = this.list(queryWrapper);
        Long passNum = list.stream().filter(s -> s.getScore() >= 90).count();
        testTotalVo.setTestNum(list.size())
                .setPassNum(passNum.intValue());
        //取最近的五次
        if (CollectionUtil.isNotEmpty(list) && list.size() > 5) {
            list = list.subList(0, 5);
        }
        List<TdQuestionTest> recentTestList = CollectionUtil.reverse(list);
        //最近五次的平均分
        OptionalDouble averaged = recentTestList.stream().mapToInt(TdQuestionTest::getScore).average();
        Integer average = 0;
        if (averaged.isPresent()){
            Double averageScore = averaged.getAsDouble();
            average = averageScore.intValue();
        }
        testTotalVo.setRecentTestList(recentTestList)
                .setRecentAverageScore(average);
        return testTotalVo;
    }

    @Override
    public List<TestScoreVO> getTestScore(TestQueryDto queryDto) {
        SecurityUser loginUser = SecurityUtil.getLoginUser();
        queryDto.setUserId(loginUser.getUserId());
        //查询当前用户的考试成绩
        List<TestScoreVO> data = this.getBaseMapper().getTestScore(queryDto);
        //查询考试配置
        List<TdSysConfig> configList = configService.queryConfigByKey("ExamParams");
        Map<Integer, String> configMap = configList.stream().collect(Collectors.toMap(TdSysConfig::getCarTypeId, TdSysConfig::getConfigJson));

        data.forEach(score -> {
            String setting = configMap.get(score.getCarTypeId());
            score.setIsPass("不合格");
            if (!StringUtils.isEmpty(setting)){
                List<ExamParam> examParams = JSONObject.parseArray(setting, ExamParam.class);
                if (examParams != null && !examParams.isEmpty()){
                    ExamParam examParam = examParams.stream().filter(d -> d.getSubject().equals(score.getSubject())).findFirst().orElse(null);
                    if (examParam != null && examParam.getOptions().getBaseScore()<= score.getScore()){
                        score.setIsPass("合格");
                    }
                }
            }
        });
        return data;
    }
}
